#include <linux/config.h>
#include <linux/version.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/timer.h>
#include <linux/skbuff.h>
#include <linux/netdevice.h>

#include <asm/rt2880/rt_mmap.h>
#include "rt_config.h"
#include "define.h"
#include "chip/rt2880.h"

#define LED_CHECK_INTERVAL 	500 /* 500ms */
#define LED_CHECK_INTERVAL_WPS 	50

#define TX_LED_BLINK_ON		30
#define TX_LED_BLINK_OFF	30

/* LED Mode */
#define LED_OFF 0
#define LED_TX_BLINK 1
#define LED_SLOW_BLINK 2
#define LED_ON 3

//#if CONFIG_RT2860V2_LED_GPIO ==9 
//#define LED_GPIO  9
//#else
//#define LED_GPIO 11
//#endif

#define LED_GPIO CONFIG_RT2860V2_LED_GPIO

#define RALINK_PRGIO_ADDR       RALINK_PIO_BASE // Programmable I/O
#define RALINK_REG_PIODATA      (RALINK_PRGIO_ADDR + 0x20)
#define RALINK_REG_PIO3924DATA          (RALINK_PRGIO_ADDR + 0x48)
#define RALINK_REG_PIO5140DATA          (RALINK_PRGIO_ADDR + 0x70)
#define RALINK_REG_PIO7140DATA          (RALINK_PRGIO_ADDR + 0x70)
#define RALINK_REG_PIO9572DATA          (RALINK_PRGIO_ADDR + 0x98)

#define RALINK_GPIO_0           0x00000001
#define RALINK_GPIO_1           0x00000002
#define RALINK_GPIO_2           0x00000004
#define RALINK_GPIO_3           0x00000008
#define RALINK_GPIO_4           0x00000010
#define RALINK_GPIO_5           0x00000020
#define RALINK_GPIO_6           0x00000040
#define RALINK_GPIO_7           0x00000080
#define RALINK_GPIO_8           0x00000100
#define RALINK_GPIO_9           0x00000200
#define RALINK_GPIO_10          0x00000400
#define RALINK_GPIO_11          0x00000800
#define RALINK_GPIO_12          0x00001000
#define RALINK_GPIO_13          0x00002000
#define RALINK_GPIO_14          0x00004000
#define RALINK_GPIO_15          0x00008000
#define RALINK_GPIO_16          0x00010000
#define RALINK_GPIO_17          0x00020000
#define RALINK_GPIO_18          0x00040000
#define RALINK_GPIO_19          0x00080000
#define RALINK_GPIO_20          0x00100000
#define RALINK_GPIO_21          0x00200000
#define RALINK_GPIO_22          0x00400000
#define RALINK_GPIO_23          0x00800000
#define RALINK_GPIO(x)          (1 << x)

#define __LED_ON(gpio) ralink_gpio_led_value |= RALINK_GPIO(gpio);
#define __LED_OFF(gpio) ralink_gpio_led_value &= ~RALINK_GPIO(gpio);

u32 ralink_gpio_led_value = 0;

typedef union _LED_CFG_T
{
	struct{
#ifdef RT_BIG_ENDIAN
		UINT32 :1;
		UINT32 LED_POL:1;		/* 0: active low, 1:active high. */
		UINT32 Y_LED_MODE:2;	/* 0: off, 1: blinking upon Tx,
									2:Periodic slow blinking, 3:sloid on. */
		UINT32 G_LED_MODE:2;	/* same as Y_LED_MODE. */
		UINT32 R_LED_MODE:2;	/* same as Y_LED_MODE. */
		UINT32 :2;
		UINT32 SLOW_BLK_TIME:6;	/* slow blinking period (uint: 1sec). */
		UINT32 LED_OFF_TIIME:8;	/* Tx blinking off period (unit: 1ms). */
		UINT32 LED_ON_TIME:8; 	/* Tx blinking on period (uint: 1ms). */
#else
		UINT32 LED_ON_TIME:8; 	/* Tx blinking on period (uint: 1ms). */
		UINT32 LED_OFF_TIME:8;	/* Tx blinking off period (unit: 1ms). */
		UINT32 SLOW_BLK_TIME:6;	/* slow blinking period (uint: 1sec). */
		UINT32 :2;
		UINT32 R_LED_MODE:2;	/* same as Y_LED_MODE. */
		UINT32 G_LED_MODE:2;	/* same as Y_LED_MODE. */
		UINT32 Y_LED_MODE:2;	/* 0: off, 1: blinking upon Tx,
									2:Periodic slow blinking, 3:sloid on. */
		UINT32 LED_POL:1;		/* 0: active low, 1:active high. */
		UINT32 :1;
#endif
	} field;
	UINT32 word;
} LED_CFG_T, *PLED_CFG_T;

RALINK_TIMER_STRUCT LedCheckTimer;
RALINK_TIMER_STRUCT LedCheckTimer_wps;

unsigned char LinkStatus;
unsigned char GPIOPolarity;
unsigned char SignalStrength;

union uLedParameter LedParameter;

extern void ralink_gpio_led_set(int gpio, unsigned int on, unsigned int off, unsigned int blinks, unsigned int rests, unsigned int times);

LED_CFG_T LedCfg;
BYTE CheckTimerEbl=0;
BYTE CheckTimerEbl_wps=0;

enum LED_STATE { SOLID_OFF=0, SOLID_ON, BLINK } led_state ;

static inline int TX_TRAFFIC_EXIST(
	IN PRTMP_ADAPTER pAd)
{
	UINT RegValue;

	RTMP_IO_READ32(pAd, MCU_INT_STATUS, &RegValue);
	return (RegValue & 0x0e);
}

static inline int BEN_TC_ACT(
	IN PRTMP_ADAPTER pAd)
{
	UINT RegValue;

	RTMP_IO_READ32(pAd, MCU_INT_STATUS, &RegValue);
	return (RegValue & 0x10);
}

// Entry for Led control.  Triggered by timer per 50ms
void LedCtrlMain(
	IN PVOID SystemSpecific1, 
	IN PVOID FunctionContext, 
	IN PVOID SystemSpecific2, 
	IN PVOID SystemSpecific3)
{
    RTMP_ADAPTER *pAd = (RTMP_ADAPTER *)FunctionContext;
    if (TX_TRAFFIC_EXIST(pAd)) // Check if there is Tx Traffic
	{
		ralink_gpio_led_set(LED_GPIO,1,1,4000,0,4000);
	}
	else
	{
		ralink_gpio_led_set(LED_GPIO,4000,0,1,0,4000);
	}

	// Clear Tx and beacon Tx complete interrupt
	
	RTMP_IO_WRITE32(pAd, MCU_INT_STATUS, 0x1e);
	
}


// Called when host sets LED mode.
// Called in interrupt level
void SetLinkStatus(IN PRTMP_ADAPTER pAd)
{
	switch(LinkStatus)
	{
		case 0x00:		// Radio off
		case 0x20:		// Radio on but not link up
			LedCfg.word = 0x0;
  			break;

		case 0x60:		// Radio on and link to G
		case 0xa0:		// Radio on and link to A
			LedCfg.field.LED_POL = 1;
			LedCfg.field.G_LED_MODE = LED_TX_BLINK;
			LedCfg.field.LED_ON_TIME = TX_LED_BLINK_ON;
			LedCfg.field.LED_OFF_TIME = TX_LED_BLINK_OFF;
  			break;

		case 0x10:		// WPS
			break;

		case 0x08:
			break;
	}

	RTMP_IO_WRITE32(pAd, LED_CFG, LedCfg.word);
    if(CheckTimerEbl==0)
    {
	    memset(&LedCheckTimer, 0, sizeof(RALINK_TIMER_STRUCT));
	    RTMPInitTimer(pAd, &LedCheckTimer, GET_TIMER_FUNCTION(LedCtrlMain), pAd, TRUE);
	    RTMPSetTimer(&LedCheckTimer, LED_CHECK_INTERVAL);
        CheckTimerEbl=1;
    }
}


void SetLinkStatusEntry(IN PRTMP_ADAPTER pAd)
{	
	SetLinkStatus(pAd);
}	// @ 0x1025

void StopLed(IN PRTMP_ADAPTER pAd)
{
	// disable WPS LED flash
	if(CheckTimerEbl_wps == 1)
	{
		BOOLEAN Cancelled;		
		RTMPCancelTimer(&LedCheckTimer_wps, &Cancelled);
		CheckTimerEbl_wps=0;
	}
	
	LedCfg.word = 0x0;
	// LED off
	RTMP_IO_WRITE32(pAd, LED_CFG, LedCfg.word);
	ralink_gpio_led_set(LED_GPIO,0,4000,0,1,4000);

}

void StartLed(IN PRTMP_ADAPTER pAd)
{
	SetLinkStatus(pAd);
}

void LedCtrlMain_wps(IN PVOID data)
{
	RALINK_TIMER_STRUCT *tmp = (RALINK_TIMER_STRUCT *) data;
	RTMP_ADAPTER *pAd = (RTMP_ADAPTER *) tmp->cookie;
	
	if (led_state != SOLID_OFF)
	{
		LedCfg.field.LED_POL = 0;
		LedCfg.field.G_LED_MODE = LED_OFF;
		led_state = SOLID_OFF;

		ralink_gpio_led_value = *(volatile u32 *)(RALINK_REG_PIODATA);
		__LED_OFF(LED_GPIO);
		*(volatile u32 *)(RALINK_REG_PIODATA) = ralink_gpio_led_value;

	} else if (led_state != SOLID_ON)
	{
		LedCfg.field.LED_POL = 0;
		LedCfg.field.G_LED_MODE = LED_ON;
		led_state = SOLID_ON;

		ralink_gpio_led_value = *(volatile u32 *)(RALINK_REG_PIODATA);
		 __LED_ON(LED_GPIO);
		 *(volatile u32 *)(RALINK_REG_PIODATA) = ralink_gpio_led_value;

	}	
	RTMP_IO_WRITE32(pAd, LED_CFG, LedCfg.word);
	
	RTMPSetTimer(&LedCheckTimer_wps, LED_CHECK_INTERVAL_WPS);
}

void FlashStartLed(IN PRTMP_ADAPTER pAd)
{
	if(CheckTimerEbl_wps == 0)
	{
		RTMPInitTimer(pAd, &LedCheckTimer_wps, LedCtrlMain_wps, pAd, FALSE);
		RTMPSetTimer(&LedCheckTimer_wps, LED_CHECK_INTERVAL_WPS);
		CheckTimerEbl_wps=1;
		led_state=SOLID_OFF;
	}
}


void FlashStopLed(IN PRTMP_ADAPTER pAd)
{
	BOOLEAN Cancelled;		
	RTMPCancelTimer(&LedCheckTimer_wps, &Cancelled);
	CheckTimerEbl_wps=0;
	SetLinkStatus(pAd);
	ralink_gpio_led_set(LED_GPIO,0,4000,0,1,4000);
}		

