
#include <rt_config.h>


/* RDS/00014 */

typedef struct {
    uint8_t  cmark;          //complete mark
    uint8_t  section_id;     //refer RDS/00018
    uint16_t magic;          //0x2B24, '$+', in little endian)
    uint32_t time_stamp;     //time stamp of section  
    uint32_t image_length;   //length of section image
    uint16_t image_checksum; //for section image
    uint16_t tag_checksum;   //for section tag
} UDG_SECTION_TAG;

#define STAG_MAGIC      0x2B24

/* RDS/00043 */
typedef struct {
    uint16_t magic;         //0x5343, 'CS' in little endian
    uint16_t checksum;      //checksum, include header & body
    uint32_t body_length;   //length of body
    uint8_t  body_encoding; //encoding method of body
    uint8_t  reserved[3];
    uint32_t raw_length;    //length of body before encoded
} CSXF_HEADER;

#define CSXF_MAGIC      		0x5343
#define CSID_S_SYS_WLAN_EEPROM	0x8000F500
#define PSECTION_SIGNATURE      0x2B240202

#ifdef CONFIG_RALINK_FLASH_API
#define PSEC_OFFSET				0x7FE000
#else
#define PSEC_OFFSET				0x00E000 // only need offset of Config //add by Aaron
extern int ra_mtd_read_nm(char *name, loff_t from, size_t len, u_char *buf);
extern int ra_mtd_write_nm(char *name, loff_t to, size_t len, const u_char *buf);
#endif

#define DEBUG 0

static unsigned char p_tmp[4096];
static unsigned char p_buf[4096];

static int p_mode,p_size;
static unsigned char *psec;

#if DEBUG
static void DUMP(unsigned char *s,int size)
{
	int m,n,i,j;
	static unsigned char buf[128];

	memset(buf,0,128);
	
	m = size / 16;
	n = size % 16;
	
	while( m )
	{
		j = 0;
		for(i=0;i<16;i++)
			j += sprintf((char*)buf+j,"%02X ",*(s+i));
		buf[j] = 0;
		printk(" %s\n",buf);
		m--;
		s += 16;
	}
	if( n )
	{
		j = 0;
        for(i=0;i<n;i++)
            j += sprintf((char*)buf+j,"%02X ",*(s+i));
		buf[j] = 0;
        printk(" %s\n",buf);
	}
}
#endif

static unsigned short chksum(unsigned short cascade_sum, unsigned char *p, int count)
{
    register unsigned short i1, i2;
	register unsigned short *cp;

	cp = (unsigned short *)p;

    i1 = cascade_sum;
    while (count--)
    {
        i2 = *cp++;
        i1 += i2;
        if (i1 < i2) i1++;
    }
    return i1;
}


static unsigned char* csid_getpos(unsigned long csid)
{
    CSXF_HEADER *csxf_header = (CSXF_HEADER*)psec;
    unsigned char *p,*pl;
    unsigned long pcsid;
    short psize;
 
    if( psec == 0 ) return 0;
    
    p = psec + 16;
    pl = psec + 16 + csxf_header->raw_length;
    
    while( p < pl )
    {
        memcpy(&pcsid,p,4);p+=4;
        memcpy(&psize,p,2);p+=2;
        if( pcsid == csid && (psize > 0) )
            return p;
        if( psize > 0 ) p += psize;
    }
    return 0;
}

static int csid_add(unsigned long csid,unsigned char *buf,unsigned short size)
{
    CSXF_HEADER *csxf = (CSXF_HEADER*)psec;
    unsigned char *p;

    if( psec == 0 ) return 0;

    p = psec + 16 + csxf->raw_length;
    p -= 4; // csid == 0
    memcpy(p,&csid,4); p += 4;  
    memcpy(p,&size,2); p += 2;  
    memcpy(p,buf,size); p += size;
    memset(p,0,4);   
    csxf->raw_length += size + 6;  
	return 1;
}


#define SEED    0x20081205


static int rebuild_p(void)
{
	CSXF_HEADER *csxf = (CSXF_HEADER*)psec;
	UDG_SECTION_TAG *stag;

//	printk("B %d\n",p_mode);

	switch( p_mode )
	{
		case 0x00:
//		#if 0 // force encode
            csxf->body_length = csxf->raw_length;
            p_size = 16 + csxf->raw_length;
			if( p_size & 1 )
				psec[p_size] = 0xff;
			csxf->body_encoding = 0x00;
			csxf->checksum = 0;
			csxf->checksum = ~chksum(0,psec,(p_size+1)>>1);
			memcpy(p_tmp+16,psec,p_size);
			break;
//		#endif
		case 0x07:
			csxf->body_length = csxf->raw_length;
			p_size = 16 + csxf->raw_length;
			if( p_size & 1 )
				psec[p_size] = 0xff;

			memcpy(p_tmp+16,psec,16); 
			jse_encode(psec+16,p_tmp+32,((csxf->body_length+3)/4), htonl(SEED));
			csxf = (CSXF_HEADER*)(p_tmp+16);
			csxf->body_encoding = 0x07;
			csxf->checksum = 0;
			if( p_size & 1 )
				*(p_tmp+16+p_size) = 0xff;
			csxf->checksum = ~chksum(0,p_tmp+16,(p_size+1)>>1);
			break;

		default:
			return 0;
    }

	// rebuild session tag
	stag = (UDG_SECTION_TAG *)p_tmp;
	stag->image_length = p_size;
	//Bizzy modify for checksum
	stag->image_checksum = chksum(0,p_tmp+16,(p_size+1)>>1);//~csxf->checksum;
	stag->tag_checksum = 0;
	stag->tag_checksum = ~chksum(0,p_tmp,16>>1);

	return 1;
}


static int check_p(unsigned char *p)
{	
	CSXF_HEADER *csxf;
	UDG_SECTION_TAG *stag;
	unsigned long csxf_size,ssid;
	unsigned short sum;

	memcpy((char*)&ssid,p,4);
	if( ssid != PSECTION_SIGNATURE )
	{
		printk("P SIG ERROR\n");
		return 0;
	}
	stag = (UDG_SECTION_TAG*)p;

	sum = ~chksum(0,p,16>>1);
	if( sum != 0 )
	{
		printk("P SEG CHKSUM ERROR\n");
		return 0;
	}

	if( stag->image_length & 1 )
		*(p+16+stag->image_length) = 0xff;

	sum = ~chksum(0,p+16,(stag->image_length+1)>>1);
	if( sum != 0 )
	{
		printk("P SEG IMG ERROR\n");
		return 0;
	}

	p += 16;	// skip session tag
	csxf = (CSXF_HEADER *)p;
	csxf_size = 16 + csxf->body_length;
    
	if ((csxf->magic != CSXF_MAGIC) || (csxf_size > (4 * 1024)))
	{
		printk("P SEG MAGIC ERROR\n");
		return 0;
	}

	p_size = csxf_size;
    if( p_size & 1 )
        p[p_size] = 0xff;
    sum = ~chksum(0,p,(p_size+1)>>1);

    if( sum != 0 )
    {
        printk("P SUM:%x\n",(int)sum);
        return 0;
    }

    p_mode = csxf->body_encoding;

	psec = p_buf;
    memcpy(p_buf,p,16);
    switch( csxf->body_encoding )
    {
		case 0x00:
            memcpy(p_buf+16,p+16,csxf->body_length);
            break;

		case 0x07:
			jse_decode(p+16, p_buf+16, ((csxf->body_length+3)/4), htonl(SEED));
            break;

        default:  
            printk("P encode=%x\n",csxf->body_encoding);
            return 0;
    }

    return 1;
}

int amit_psec_read(unsigned char *buf,int size)
{
	unsigned char *eeprom;

#ifdef CONFIG_RALINK_FLASH_API
        FlashRead((uint16_t *)p_tmp, PSEC_OFFSET, 4096);
#else
        ra_mtd_read_nm("Config", PSEC_OFFSET, 4096, p_tmp);
#endif

	#if DEBUG
	printk("read p section ==>\n");
	DUMP(p_tmp,200);
	printk("<===read p section \n");
	#endif
	
	psec = 0;
	if( !check_p(p_tmp) )
		return 0;

	eeprom = csid_getpos(CSID_S_SYS_WLAN_EEPROM);
	if( eeprom == 0 ) 
	{
		printk("P No EEPROM\n");
		return 0;
	}

	#if DEBUG
	DUMP(eeprom,size);
	#endif

	memcpy(buf,eeprom,size);
	return 1;
}


int amit_psec_write(unsigned char *buf,int size)
{
	unsigned char *ofs;
	int i;

//check P section first // add by aaron to fix As not P section ,and wifi want to write configuration
#ifdef CONFIG_RALINK_FLASH_API
        FlashRead((uint16_t *)p_tmp, PSEC_OFFSET, 4096);
#else
        ra_mtd_read_nm("Config", PSEC_OFFSET, 4096, p_tmp);
#endif	
//	ra_mtd_read_nm("Config", PSEC_OFFSET, 4096, p_tmp);
	
	if( !check_p(p_tmp) )
		return 0;
	ofs = csid_getpos(CSID_S_SYS_WLAN_EEPROM);

	if( ofs ) 
        memcpy(ofs,buf,size);
	else
		csid_add(CSID_S_SYS_WLAN_EEPROM,buf,size);

	rebuild_p();
	
	#if DEBUG
	printk("after rebuild_p ===>\n");
	DUMP(p_tmp,p_size+16);
	printk("<=== after rebuild_p \n");
	#endif

#ifdef CONFIG_RALINK_FLASH_API
        FlashWrite((uint16_t *)p_tmp, PSEC_OFFSET, 4096);
#else
        ra_mtd_write_nm("Config", PSEC_OFFSET, 4096, p_tmp);
#endif

	return 1;
}

