/****************************************************************************
* File:         $Workfile$
* Revision:     $Revision: 1.6.4.1.2.1 $
* Date:         $Date: 2010-05-13 08:20:48 $
*
* Overview:     
* Functions:
*
* Author:       Ian Tsao
* 
* -- Metanoia Copyright Notice --
*
* (C) Copyright 2005 Metanoia, Inc. All rights reserved.
* 
* Metanoia reserves the right to change specifications without notice. 
* Copyright 2005 Metanoia, Inc. All rights reserved. Preliminary Data-Sheet 
* indicates this product is in design and the specification may change. 
* Electrical parametrics have not been analyzed and are not specified. Do not 
* use this data sheet as a design reference document. Please contact Metanoia 
* for a current data sheet that may be used as a design reference. All 
* contents of this document are protected by copyright law and may not be 
* reproduced without the express written consent of Metanoia, Inc. Metanoia, 
* the Metanoia logo, and combinations thereof are trademarks of Metanoia, Inc. 
* Other product names used in this publication are for identification purposes 
* only and may be trademarks or registered trademarks of their respective 
* companies. The contents of this document are provided in connection with 
* Metanoia, Inc. products. Metanoia, Inc. has made best efforts to ensure that 
* the information contained herein is accurate and reliable. However, Metanoia, 
* Inc. makes no warranties, express or implied, as to the accuracy or 
* completeness of the contents of this publication and is providing this 
* publication 'as is'. Metanoia, Inc. reserves the right to make changes to 
* specifications and product descriptions at any time without notice, and to 
* discontinue or make changes to its products at any time without notice. 
* Metanoia, Inc. does not assume any liability arising out of the application 
* or use of any product or circuit, and specifically disclaims any and all 
* liability, including without limitation special, consequential, or 
* incidental damages.
* 
****************************************************************************/
#ifndef _VDSLLIB_H_
#define _VDSLLIB_H_

#include "userdef.h"

/**
 * \file 
 * \brief API set that metanoia provide
 */
 
// -------------------------- System related  -----------------------

#define MT_OS_LINUX 

#if !defined(MT_OS_TYPE)
#	define MT_OS_TYPE MT_OS_LINUX
#endif


//Notification that can be enable/disable
#define MT_NOTIFICATION_VdslDownMaxSnrMgnNotification			8
#define MT_NOTIFICATION_VdslDownMinSnrMgnNotification			9
#define MT_NOTIFICATION_VdslUpMaxSnrMgnNotification				10
#define MT_NOTIFICATION_VdslUpMinSnrMgnNotification				11
#define MT_NOTIFICATION_VdslInitFailureNotification				12
#define MT_NOTIFICATION_VdslOpState								13


// -------------------------- Misc -----------------------
#define VDSL_CHIPID 1

//#define GETDATA(portid, oid, value) (get_port_mib(portid, oid, value))
//#define SETDATA(portid, oid, value) (set_port_mib(portid, oid, value))

// -------------------------- Performance -----------------------
#define MT_LINE_SIZE15 9
#define MT_LINEINIT_SIZE15 7
#define MT_CHAN_SIZE15 6
#define MT_LINE_SIZE24 9
#define MT_LINEINIT_SIZE24 7
#define MT_CHAN_SIZE24 6

extern mt_uint32 default_alarm_profile[9];
extern mt_uint32 default_profile[45];

//< -------------------------- Performance -----------------------
/** 
 * performance related define
 */
#define MT_PERF_1SEC 0
#define MT_PERF_15MIN 1
#define MT_PERF_24HOUR 2

// performance interval
#define MT_PERF_CURR 0
#define MT_PERF_PREV 1

//< -------------------------- MIB -----------------------
// MT_MIB_VdslTerminal - bit fields
//Bit0(1-RT,0-OT) Bit1(1-Xenos/SMII,0-Award/MII) Bit2(1-HDLC,0-NOHDLC) Bit3(1-NeedConnect,0-NoNeedConnect) Bit4(1-with pacing signal,0-w/o pacing signal)
//Bit8 & Bit9(3-Showtime,2-Training,1-Handshake,0-Idle)
#define MT_MIB_RTOT										0
#define MT_MIB_XenosSMIIAwardMII						1
#define MT_MIB_HDLC										2
#define MT_MIB_NeedConnect								3
#define MT_MIB_PacingSignal								4
#define MT_MIB_HandshakeIdle							8
#define MT_MIB_ShowtimeTraining							9

// MT_MIB_VdslTerminal Bit8 & Bit9 representation
#define IDLE 0
#define HANDSHAKE 1
#define TRAINING 2
#define SHOWTIME 3

typedef struct 
{
	float TarSnrMarDs;
	float TarSnrMarUs;
	float MaxSnrMarDs;
	float MaxSnrMarUs;
	float MinSnrMarDs;
	float MinSnrMarUs;
}snr_mar_struct;

typedef struct 
{
	mt_uint32 EsLCntrs;
	mt_uint32 SesLCntrs;
	mt_uint32 LossLCntrs;
	mt_uint32 UasLCntrs;
}line_perf_cntr_struct;

// -------------------------- Profile -----------------------
// profile name storage buffer
//extern char profile_name_list[MAX_PORTS][MT_MIB_ConfProfileNameMaxLength];

typedef struct
{
	/** 
	 *  dslLib support profile ID, but not profile Name
	 *  AP need to have mapping with Name and ID if they need to have profile name
	 */
	
	/// 
	union conf_union{
		mt_uint32		buf[46];
		struct {
	    mt_uint32		vdslLineConfProfileName;
	    mt_uint32		vdslLineConfDownRateMode;
	    mt_uint32		vdslLineConfUpRateMode;
	    mt_uint32		vdslLineConfDownMaxPwr;
	    mt_uint32		vdslLineConfUpMaxPwr;
	    mt_uint32		vdslLineConfDownMaxSnrMgn;
	    mt_uint32		vdslLineConfDownMinSnrMgn;
	    mt_uint32		vdslLineConfDownTargetSnrMgn;
	    mt_uint32		vdslLineConfUpMaxSnrMgn;
	    mt_uint32		vdslLineConfUpMinSnrMgn;
	    mt_uint32		vdslLineConfUpTargetSnrMgn;
	    mt_uint32		vdslLineConfDownFastMaxDataRate;
	    mt_uint32		vdslLineConfDownFastMinDataRate;
	    mt_uint32		vdslLineConfDownSlowMaxDataRate;
	    mt_uint32		vdslLineConfDownSlowMinDataRate;
	    mt_uint32		vdslLineConfUpFastMaxDataRate;
	    mt_uint32		vdslLineConfUpFastMinDataRate;
	    mt_uint32		vdslLineConfUpSlowMaxDataRate;
	    mt_uint32		vdslLineConfUpSlowMinDataRate;
	    mt_uint32		vdslLineConfDownRateRatio;
	    mt_uint32		vdslLineConfUpRateRatio;
	    mt_uint32		vdslLineConfDownMaxInterDelay;
	    mt_uint32		vdslLineConfUpMaxInterDelay;
	    mt_uint32		vdslLineConfDownPboControl;
	    mt_uint32		vdslLineConfUpPboControl;
	    mt_uint32		vdslLineConfDownPboLevel;
	    mt_uint32		vdslLineConfUpPboLevel;
	    mt_uint32		vdslLineConfDeploymentScenario;
	    mt_uint32		vdslLineConfAdslPresence;
	    mt_uint32		vdslLineConfApplicableStandard;
	    mt_uint32		vdslLineConfBandPlan;
	    mt_uint32		vdslLineConfBandPlanFx;
	    mt_uint32		vdslLineConfBandOptUsage;
	    mt_uint32		vdslLineConfUpPsdTemplate;
	    mt_uint32		vdslLineConfDownPsdTemplate;
	    mt_uint32		vdslLineConfHamBandMask;
	    mt_uint32		vdslLineConfCustomNotch1Start;
	    mt_uint32		vdslLineConfCustomNotch1Stop;
	    mt_uint32		vdslLineConfCustomNotch2Start;
	    mt_uint32		vdslLineConfCustomNotch2Stop;
	    mt_uint32		vdslLineConfDownTargetSlowBurst;
	    mt_uint32		vdslLineConfUpTargetSlowBurst;
	    mt_uint32		vdslLineConfDownMaxFastFec;
	    mt_uint32		vdslLineConfUpMaxFastFec;
	    mt_uint32		vdslLineConfLineType;
	    mt_uint32		vdslLineConfProfRowStatus;
		}par;
	}conf;
}conf_profile_struct;

typedef struct
{
	union alarm_union{
		mt_uint32	buf[10];
		struct {
			mt_uint32		vdslLineAlarmConfProfileName;
    		mt_uint32		vdslLineAlarmConfThresh15MinLofs;
    		mt_uint32		vdslLineAlarmConfThresh15MinLoss;
    		mt_uint32		vdslLineAlarmConfThresh15MinLprs;
    		mt_uint32		vdslLineAlarmConfThresh15MinLols;
    		mt_uint32		vdslLineAlarmConfThresh15MinESs;
    		mt_uint32		vdslLineAlarmConfThresh15MinSESs;
    		mt_uint32		vdslLineAlarmConfThresh15MinUASs;
    		mt_uint32		vdslLineAlarmConfInitFailure;
    		mt_uint32		vdslLineAlarmConfProfRowStatus;     
		}par;
	}conf;
}alarm_profile_struct;

// -------------------------- Callback, Alarm -----------------------



/**
 * the interface to call user when notificaion is needed.
 * 
 */
typedef struct
{
	mt_uint32 portid;
	mt_uint32 nid; ///< Notification ID, multi-notification in this value
	mt_uint32 nstatus; ///< MT_MIB_VdslPhysCurrStatus's notification
	
}notification_data;

typedef int (*notification_callback)(notification_data *ns);

typedef struct
{
	notification_callback nif;	//< Notification Callback Function Pointer
	int npid;	//< Notification Process ID
}notification_param;

#ifdef __KERNEL__
typedef struct notification_link{
	struct	list_head	link;	/* double linked list */
	notification_param	param;	/* notification data */
}notification_link_t;

static LIST_HEAD(notification_linkq);
#else
notification_callback notification_intrfce;
#endif
// -------------------------- Function declaration -----------------------
// following function implementation in if.c
//Init Function
mt_ret init_system(void);
mt_ret init_chip(mt_uint32 chipid, mt_uint32 firmwareID);
mt_ret init_modem(mt_uint32 chipid);
mt_ret reset_modem(mt_uint32 chipid);
mt_ret download_firmware(mt_uint8 chipid, mt_uint8 cmdID, mt_uint16 bufsize,mt_uint8* buffer);
//Port Function
mt_ret connect_port(mt_uint32 portid);
mt_ret disconnect_port(mt_uint32 portid);
mt_ret clear_port_stats(mt_uint32 portid);
//Shutdown Function
mt_ret shutdown_system(void);
mt_ret shutdown_chip(mt_uint32 chipid);
//System Function
mt_ret get_system_info(sys_info_struct *psi);
mt_ret get_alarms(alarm_struct* palarm);
mt_ret clear_alarms(void);
//Diagnostic Funtions
mt_ret system_run_diag_tests(void);
mt_ret chip_run_diag_tests(void);
mt_ret port_run_diag_tests(void);					  
// following function implementation in vdsllib.c
// -------------------------- MIB  -----------------------
mt_ret get_port_mib(mt_uint32 portid, mt_uint8 *loid, mt_uint8 *strvalue);
mt_ret set_port_mib(mt_uint32 portid, mt_uint8 *loid, mt_uint8 *strvalue);
#ifndef CHIP_F4
mt_ret get_port_mib_array(mt_uint32 portid,mt_uint32 groupid,mt_uint32 tableid,mt_uint32 idx, mt_uint32 paramid,mt_uint32 interval,mt_uint32 count, mt_uint32* value);
mt_ret set_port_mib_array(mt_uint32 portid, mt_uint32 idx, mt_uint32 count_of_array, mib_value_set *mib_array);
#endif
mt_ret get_phy_status(mt_uint32 portid, mt_uint8 *phy_status, mt_uint8 *conn_status);
mt_ret get_op_status(mt_uint32 portid,mt_uint8 *state);
// ---------------------------- Conf Profile Management -------------------
#ifndef CHIP_F4
mt_ret init_conf_profile(mt_uint32 portid,conf_profile_struct *ps);
mt_ret get_conf_profile(mt_uint32 portid,conf_profile_struct *ps);
mt_ret set_conf_profile(mt_uint32 portid, conf_profile_struct *ps);
// ---------------------------- Alarm Profile Management -------------------
mt_ret init_alarm_conf_profile(mt_uint32 portid,alarm_profile_struct *ps);
mt_ret set_alarm_profile(mt_uint32 portid,alarm_profile_struct *ps);
mt_ret get_alarm_profile(mt_uint32 portid,alarm_profile_struct *ps);
// -------------------------- Notification -----------------------
mt_ret setup_notification(notification_param nparam);
mt_ret remove_notification(notification_param nparam);
mt_ret mask_notification(mt_uint32 nid,mt_uint32 nstatus);
mt_ret process_notification(void);
#endif	//CHIP_F4
// -------------------------- Statistic Routine -----------------------
#ifndef CHIP_F4
mt_ret mt_perf_15min(void);
mt_ret mt_perf_24hour(void);
mt_ret clear_port_stats(mt_uint32 portid);
mt_ret statistic_trigger(void);
#endif	//CHIP_F4
//-------------------------- Debug ----------------------- */
//mt_ret enable_debug(void);
//mt_ret disable_debug(void);
// -------------------------- Misc -----------------------
mt_ret init_port_info(void);
// -------------------------- Timer -----------------------
#ifdef __KERNEL__
#ifndef CHIP_F4
void activate_timer1(void);
void deactivate_timer1(void);
void eoc_timer_handler(unsigned long chipid);
void activate_timer2(void);
void deactivate_timer2(void);
#endif	//CHIP_F4
int statistic_thread_handler(void *dummy);
#ifdef KERNEL_2_6
#define MSLEEP msleep
#else
#define MSLEEP mysleep
#define CLONE_KERNEL (CLONE_FS | CLONE_FILES | CLONE_SIGHAND)
extern void mysleep(unsigned long msecs);
#endif //KERNEL_2_6
#define MYDELAY mdelay
//#define MYDELAY MSLEEP
#endif	//__KERNEL__

#endif //_VDSLLIB_H_
