/*
 * @file Member ID Definition of Group REMOTE.
 *
 * Moderator: Aaron Wu
 * Group ID: 0x00760000/0x80760000
 */

#ifndef _CSID_REMOTE_H
#define _CSID_REMOTE_H

#include "csid_gid.h"

#define _CFG_REMOTE(x)       (CSID_GIDC_REMOTE|(x))
#define _STA_REMOTE(x)       (CSID_GIDS_REMOTE|(x))

/*TRAP Message sender*/
#define CSID_C_REMOTE_AP_CONTROLLER_ENABLE		_CFG_REMOTE(0x0000)  //T=u16, AP controller main enable, 0:disable, 1:enable
#define CSID_C_REMOTE_AP_TRAP_ENABLE                    _CFG_REMOTE(0x0001)  //T=u16, AP trap messgae enable, 0:disable, 1:enable
#define CSID_C_REMOTE_AP_TRAP_ALERT_L1_ENABLE           _CFG_REMOTE(0x0002)  //T=u16, AP trap alert Levet 1 enable or disable, 0: disable, 1:enable
#define CSID_C_REMOTE_AP_TRAP_ALERT_L2_ENABLE           _CFG_REMOTE(0x0003)  //T=u16, AP trap alert Levet 2 enable or disable, 0: disable, 1:enable
#define CSID_C_REMOTE_AP_TRAP_ALERT_L3_ENABLE           _CFG_REMOTE(0x0004)  //T=u16, AP trap alert Levet 3 enable or disable, 0: disable, 1:enable
#define CSID_C_REMOTE_AP_TRAP_ALERT_L4_ENABLE           _CFG_REMOTE(0x0005)  //T=u16, AP trap alert Levet 4 enable or disable, 0: disable, 1:enable
#define CSID_C_REMOTE_AP_TRAP_ALERT_L5_ENABLE           _CFG_REMOTE(0x0006)  //T=u16, AP trap alert Levet 5 enable or disable, 0: disable, 1:enable
#define CSID_C_REMOTE_AP_TRAP_AP_TRAFFIC_ENABLE         _CFG_REMOTE(0x0007)  //T=u16, AP trap Tx Rx Packet enable
#define CSID_C_REMOTE_AP_TRAP_USER_TRAFFIC_ENABLE       _CFG_REMOTE(0x0008)  //T=u16, AP trap users upload, download traffic enable,disable
#define CSID_C_REMOTE_AP_TRAP_USER_TXRX_ENABLE          _CFG_REMOTE(0x0009)  //T=u16, AP trap users Tx Rate, Rx Raite traffic enable,disable
/*Trap Message sender Counter*/
#define CSID_C_REMOTE_AP_TRAP_ALERT_L1_COUNTER		_CFG_REMOTE(0x0100)  //T=u32, AP trap alert Level 1 counter
#define CSID_C_REMOTE_AP_TRAP_ALERT_L2_COUNTER          _CFG_REMOTE(0x0101)  //T=u32, AP trap alert Level 2 counter
#define CSID_C_REMOTE_AP_TRAP_ALERT_L3_COUNTER          _CFG_REMOTE(0x0102)  //T=u32, AP trap alert Level 3 counter
#define CSID_C_REMOTE_AP_TRAP_ALERT_L4_COUNTER          _CFG_REMOTE(0x0103)  //T=u32, AP trap alert Level 4 counter
#define CSID_C_REMOTE_AP_TRAP_ALERT_L5_COUNTER          _CFG_REMOTE(0x0104)  //T=u32, AP trap alert Level 5 counter
#define CSID_C_REMOTE_AP_TRAP_TRAFFIC_COUNTER		_CFG_REMOTE(0x0105)  //T=u32, D=5, AP trap traffic static counter

/*AP Function config seting*/
#define CSID_C_REMOTE_AP_ALLOW_DENY_ENABLE		_CFG_REMOTE(0x0106)  //T=u8, D=0, set AP allow or deny, 0: allow, 1:deny

//Trust AP Items	
#define CSID_S_REMOTE_AP_TRUSTAP_NAME                   _STA_REMOTE(0x0000)  //T=str, trust AP name
#define CSID_S_REMOTE_AP_TRUSTAP_IP                     _STA_REMOTE(0x0080)  //T=ipv4, trust AP IP Address
#define CSID_S_REMOTE_AP_TRUSTAP_MAC                    _STA_REMOTE(0x0100)  //T=mac, trust AP MAC Address
#define CSID_S_REMOTE_AP_TRUSTAP_SSID_AP1               _STA_REMOTE(0x0180)  //T=str, trust AP 2.4G SSID name
#define CSID_S_REMOTE_AP_TRUSTAP_SSID_AP2               _STA_REMOTE(0x0200)  //T=str, trust AP 5G SSID name
#define CSID_S_REMOTE_AP_TRUSTAP_STATUS                 _STA_REMOTE(0x0280)  //T=u8, trust AP status, 0:offline , 1:online, 2:denined
#define CSID_S_REMOTE_AP_TRUSTAP_TRAFFIC                _STA_REMOTE(0x0300)  //T=u32, trust AP traffic statistic
#define CSID_S_REMOTE_AP_TRUSTAP_VID_LOCATION_AP1       _STA_REMOTE(0x0380)  //T=u16, trust AP VID location
#define CSID_S_REMOTE_AP_TRUSTAP_VID_LOCATION_AP2       _STA_REMOTE(0x0400)  //T=u16, trust AP VID location
#define CSID_S_REMOTE_AP_TRUSTAP_CHANNEL_AP1            _STA_REMOTE(0x0480)  //T=u16, trust AP 2.4G channel
#define CSID_S_REMOTE_AP_TRUSTAP_CHANNEL_AP2            _STA_REMOTE(0x0500)  //T=u16, trust AP 5G channel
#define CSID_S_REMOTE_AP_TRUSTAP_ASSOICATED_AP1         _STA_REMOTE(0x0580)  //T=u32, trust AP 2.4G Assoicated station number
#define CSID_S_REMOTE_AP_TRUSTAP_ASSOICATED_AP2         _STA_REMOTE(0x0600)  //T=u32, trust AP 5G Assoicated station number
#define CSID_S_REMOTE_AP_TRUSTAP_CPU_USED               _STA_REMOTE(0x0680)  //T=str, trust AP CPU Used
#define CSID_S_REMOTE_AP_TRUSTAP_MEMORY_USED            _STA_REMOTE(0x0700)  //T=u32, trust AP Memory Used
#define CSID_S_REMOTE_AP_TRUSTAP_ROM_ID			_STA_REMOTE(0x0780)  //T=str, trust AP ROM ID
#define CSID_S_REMOTE_AP_TRUSTAP_PARSED			_STA_REMOTE(0x0800)  //T=u16, record which trust AP is get
#define CSID_S_REMOTE_AP_TRUSTAP_TX_MBPS		_STA_REMOTE(0x0880)  //T=u32, record trust AP 2.4G+5G Upload Mbps
#define CSID_S_REMOTE_AP_TRUSTAP_RX_MBPS		_STA_REMOTE(0x0900)  //T=u32, record trust AP 2.4G+5G Download Mbps
#define CSID_S_REMOTE_AP_TRUSTAP_TXRX_MBPS		_STA_REMOTE(0x0980)  //T=u32, record trust AP 2.4G+5G Totoal Mbps
#define CSID_S_REMOTE_AP_TRUSTAP_UPTIME			_STA_REMOTE(0x0a00)  //T=str, record trust AP Working time
#define CSID_S_REMOTE_AP_TRUSTAP_TRAP_TIME		_STA_REMOTE(0x0a80)  //T=str, record trust AP sending trap message time
#define CSID_S_REMOTE_AP_TRUSTAP_TRAP_STIME		_STA_REMOTE(0x0b00)  //T=str, record trust AP sending trap message time for string format, e.g. Wed Jul 7 15:37:53 2010
#define CSID_S_REMOTE_AP_TRUSTAP_CUSTOM_NAME		_STA_REMOTE(0x0b80)  //T=str, record custom name for using replace CSID_S_REMOTE_AP_TRUSTAP_NAME
#define CSID_S_REMOTE_AP_TRUSTAP_ALLOW_DENY		_STA_REMOTE(0x0c00) //T=u32, show AP If allow or deny

//Known AP Items
#define CSID_S_REMOTE_AP_KNOWNAP_IP                	_STA_REMOTE(0x2000)  //T=ipv4, known AP IP Address
#define CSID_S_REMOTE_AP_KNOWNAP_MAC                	_STA_REMOTE(0x2180)  //T=mac, known AP MAC Address
#define CSID_S_REMOTE_AP_KNOWNAP_SSID_AP1               _STA_REMOTE(0x2200)  //T=str, known AP 2.4G SSID name
#define CSID_S_REMOTE_AP_KNOWNAP_SSID_AP2               _STA_REMOTE(0x2280)  //T=str, known AP 5G SSID name
#define CSID_S_REMOTE_AP_KNOWNAP_STATUS               	_STA_REMOTE(0x2300)  //T=u8, known AP status, 0:offline , 1:online, 2:denined
#define CSID_S_REMOTE_AP_KNOWNAP_VID_LOCATION_AP1       _STA_REMOTE(0x2380)  //T=u16, known AP VID location
#define CSID_S_REMOTE_AP_KNOWNAP_VID_LOCATION_AP2       _STA_REMOTE(0x2400)  //T=u16, known AP VID location
#define CSID_S_REMOTE_AP_KNOWNAP_CHANNEL_AP1            _STA_REMOTE(0x2480)  //T=u16, known AP 2.4G channel
#define CSID_S_REMOTE_AP_KNOWNAP_CHANNEL_AP2            _STA_REMOTE(0x2500)  //T=u16, known AP 5G channel
//Rouge AP Items
#define CSID_S_REMOTE_AP_ROUGEAP_MAC                	_STA_REMOTE(0x3000)  //T=mac, rouge AP MAC Address
#define CSID_S_REMOTE_AP_ROUGEAP_SSID_AP1              	_STA_REMOTE(0x3080)  //T=str, rouge AP 2.4G SSID name
#define CSID_S_REMOTE_AP_ROUGEAP_SSID_AP2              	_STA_REMOTE(0x3100)  //T=str, rouge AP 5G SSID name
#define CSID_S_REMOTE_AP_ROUGEAP_STATUS              	_STA_REMOTE(0x3180)  //T=u8, rouge AP status, 0:offline , 1:online, 2:denined
#define CSID_S_REMOTE_AP_ROUGEAP_VID_LOCATION_AP1       _STA_REMOTE(0x3200)  //T=u16, rouge AP VID location
#define CSID_S_REMOTE_AP_ROUGEAP_VID_LOCATION_AP2       _STA_REMOTE(0x3280)  //T=u16, rouge AP VID location
#define CSID_S_REMOTE_AP_ROUGEAP_CHANNEL_AP1            _STA_REMOTE(0x3300)  //T=u16, known AP 2.4G channel
#define CSID_S_REMOTE_AP_ROUGEAP_CHANNEL_AP2            _STA_REMOTE(0x3380)  //T=u16, known AP 5G channel

/*Proposal Items*/
#define CSID_S_REMOTE_AP_PROPOSAL_ROM_ID		_STA_REMOTE(0x4000)	 //T=str, Save rom id for create proposal file
#define CSID_S_REMOTE_AP_PROPOSAL_SAVE_NAME		_STA_REMOTE(0x4100)	 //T=str, Save all proposal file name
#define CSID_S_REMOTE_AP_PROPOSAL_GET_SAVE_NAME		_STA_REMOTE(0x4300)	 //T=str, Save get AP Proposal name
#define CSID_S_REMOTE_AP_PROPOSAL_SAVE_ROM_ID		_STA_REMOTE(0x4400)	 //T=str, Save rom id for PROPOSAL_SAVE_NAME 
#define CSID_S_REMOTE_AP_PROPOSAL_SAVE_NAME_WIFIBAND	_STA_REMOTE(0x4500)	 //T=u8, indicate proposal file support band, 0: none , 1:2.4G , 2:5G, 3:dual
/* SNMP Setting Items */
//Basic	
#define CSID_S_REMOTE_AP_SNMP_GET_COMMUNITY         	_STA_REMOTE(0x5000)  //T=str, AP SNMP Get Community
#define CSID_S_REMOTE_AP_SNMP_SET_COMMUNITY         	_STA_REMOTE(0x5080)  //T=str, AP SNMP Set Community
#define CSID_S_REMOTE_AP_SNMP_VERSION_V1            	_STA_REMOTE(0x5100)  //T=u16, AP SNMP V1 Enable
#define CSID_S_REMOTE_AP_SNMP_VERSION_V2C           	_STA_REMOTE(0x5180)  //T=u16, AP SNMP V2C Enable
#define CSID_S_REMOTE_AP_SNMP_VERSION_V3            	_STA_REMOTE(0x5200)  //T=u16, AP SNMP V3 Enable
//SNMP V3
#define CSID_S_REMOTE_AP_SNMP_V3_USER_ENABLE        	_STA_REMOTE(0x5280)  //T=u16, AP SNMP v3 Enable Multi Account, 1:yes,0:no  
#define CSID_S_REMOTE_AP_SNMP_V3_USER              	_STA_REMOTE(0x5300)  //T=str, AP SNMP v3 User Account
#define CSID_S_REMOTE_AP_SNMP_V3_PASSWORD           	_STA_REMOTE(0x5380)  //T=str, AP SNMP v3 User Password
#define CSID_S_REMOTE_AP_SNMP_V3_AUTHTYPE          	_STA_REMOTE(0x5400)  //T=u16, AP SNMP v3 Auth Type, 0:MD5,1:SHA 
#define CSID_S_REMOTE_AP_SNMP_V3_PRIVACYMODE        	_STA_REMOTE(0x5480)  //T=u16, AP SNMP v3 Privacy Mode, 0:noAuthNoPriv,1:authNoPriv,2:authPriv
#define CSID_S_REMOTE_AP_SNMP_V3_PRIVACY           	_STA_REMOTE(0x5500)  //T=str, AP SNMP v3 DES passphrase 
#define CSID_S_REMOTE_AP_SNMP_V3_RO_RW              	_STA_REMOTE(0x5580)  //T=str, AP SNMP v3 User RO/RW Enable, 0:read only,1:read write

/*AP Mode LAN */
#define CSID_S_REMOTE_AP_LAN_IP				_STA_REMOTE(0x6000)  //T=ipv4, AP mode LAN IP address			
#define CSID_S_REMOTE_AP_LAN_NM				_STA_REMOTE(0x6001)  //T=ipv4, AP mode LAN Subnet Mask
#define CSID_S_REMOTE_AP_CNT_TYPE			_STA_REMOTE(0x6002)  //T=u16, AP mode Connect type, 0:DHCP, 1:Static
#define CSID_S_REMOTE_AP_FIX_LAN_IP			_STA_REMOTE(0x6003)  //T=ipv4, AP mode FIX LAN IP
#define CSID_S_REMOTE_AP_FIX_LAN_NM			_STA_REMOTE(0x6004)  //T=ipv4, AP mode FIX LAN Subnet Mask
#define CSID_S_REMOTE_AP_FIX_GW				_STA_REMOTE(0x6005)  //T=ipv4, AP mode FIX LAN Gateway
#define CSID_S_REMOTE_AP_FIX_PRIDNS			_STA_REMOTE(0x6006)  //T=ipv4, AP mode FIX LAN Primary DNS
#define CSID_S_REMOTE_AP_FIX_SECDNS			_STA_REMOTE(0x6007)  //T=ipv4, AP mode FIX LAN Secondary DNS

/*AP Traffic 2.4G+5G Mbps*/
#define CSID_S_REMOTE_AP_TX_MBPS							_STA_REMOTE(0x6100)	 //T=str, AP 2.4G+5G Traffic Tx Mbps
#define CSID_S_REMOTE_AP_RX_MBPS							_STA_REMOTE(0x6101)	 //T=str, AP 2.4G+5G Traffic Rx Mbps
#define CSID_S_REMOTE_AP_TXRX_MBPS							_STA_REMOTE(0x6102)  //T=str, AP 2.4G+5G Traffic Tx+Rx Mbps

#endif //ifndef _CSID_REMOTE_H


