/*
 * @file Member ID Definition of Group WEBUI.
 *
 * Moderator: SerenaWang
 * Group ID: 0x00200000
 */

#ifndef _CSID_WEBUI_H
#define _CSID_WEBUI_H

#include "csid_gid.h"

#define _CFG_WEBUI(x)         (CSID_GIDC_WEBUI|(x))
#define _STA_WEBUI(x)         (CSID_GIDS_WEBUI|(x))

#define FW_QUERY_STOP			0x00
#define FW_QUERYING 			0x01
#define FW_QUERY_DONE 		    0x02
#define FW_DOWNLOADING		    0x03
#define FW_DOWNLOAD_FAIL	    0x04
#define FW_DOWNLOAD_DONE	    0x05
#define FW_UPGRADING			0x06
#define FW_UPGRADE_DONE		    0x07
#define FW_UPGRADE_FAIL		    0x08

/* Firmware Upgrade Status */
#define FWUPG_ACTION_NONE       0x00
#define FWUPG_DOWNLOAD_START    0x01
#define FWUPG_DOWNLOAD_DONE     0x02
#define FWUPG_DOWNLOAD_FAIL     0x03
#define FWUPG_UPGRADE_START     0x04
#define FWUPG_UPGRADE_DONE      0x05
#define FWUPG_UPGRADE_FAIL      0x06

/* Configuration Items */

//==========================================================
// 0x0000~0x0FFF: Reserved.
//==========================================================

#define CSID_C_WEBUI_WIZARD_EXECED            _CFG_WEBUI(0x1001)	//T=u8, D=0, 1:yes 0:no
#define CSID_C_WEBUI_WIZARD_TMP_LANIP         _CFG_WEBUI(0x1002)	//T=ipv4, D={192.168.123.254}, lan ip
#define CSID_C_WEBUI_FW_QUERY_AUTO_ENABLE     _CFG_WEBUI(0x1003)  //T=u8, enable auto fw query
#define CSID_C_WEBUI_FW_QUERY_LAST_TIME       _CFG_WEBUI(0x1004)  //T=u32, fw query's last ntp time
#define CSID_C_WEBUI_FW_QUERY_TIME            _CFG_WEBUI(0x1005)  //T=u32, auto fw query's time(sec)
#define CSID_C_WEBUI_FW_QUERY_REGION_NAME     _CFG_WEBUI(0x1006)  //T=str, the region name of fw query
#define CSID_C_WEBUI_FW_QUERY_VER             _CFG_WEBUI(0x1007)  //T=str, the fw version of fw query
#define CSID_C_WEBUI_FW_QUERY_VER_MATCH       _CFG_WEBUI(0x1008)  //T=u32, the matching flag between current fw version and fw query version
#define CSID_C_WEBUI_FW_QUERY_DATE            _CFG_WEBUI(0x1009)  //T=str, the release date of fw query
#define CSID_C_WEBUI_FW_QUERY_DW_URL          _CFG_WEBUI(0x100a)  //T=str, the fw download url of fw query
#define	CSID_C_WEBUI_FAKE_WEPKEYLENGTH 	      _CFG_WEBUI(0x100b)	//T=u8, D=0, D-link DWR927 WEP Key Lengh is fake
#define	CSID_C_WEBUI_PASS_CHANGED     	      _CFG_WEBUI(0x100c)	//T=u8, D=0, Zyxel change password flag
#define	CSID_C_WEBUI_VLAN_TAGBASE_FLAG 	      _CFG_WEBUI(0x100d)	//T=u8, D=0, nobrand UI tagbase flag
#define	CSID_C_WEBUI_DHCP_FLAG 	              _CFG_WEBUI(0x100e)	//T=u8, D=0, VPN PPTP total switch
#define	CSID_C_WEBUI_VPNPPTP_ENABLE 	        _CFG_WEBUI(0x100f)	//T=u8, D=0, VPN L2TP total switch
#define	CSID_C_WEBUI_VPNL2TP_ENABLE 	        _CFG_WEBUI(0x1010)	//T=u8, D=0, DHCP SERVER UI flag
#define CSID_C_WEBUI_VLAN_GROUP_NAME          _CFG_WEBUI(0x1011)  //T=str, the Zyxel VLAN Group name,reserve 8
#define	CSID_C_WEBUI_ROUTER_TYPE    	        _CFG_WEBUI(0x1012)	//T=u8, D=0, 0:wireless router 1:AP router 2:wired router
#define CSID_C_WEBUI_VLAN_PORT_NUM            _CFG_WEBUI(0x1019)  //T=u8, D=5, AMIT VLAN PORT NUMBER
#define CSID_C_WEBUI_VLAN_VAP_NUM             _CFG_WEBUI(0x101a)  //T=u8, D=8, AMIT VLAN VAP NUMBER
#define CSID_C_WEBUI_VLAN_TAG_INDEX           _CFG_WEBUI(0x101b)  //T=u8, D=0, AMIT VLAN TAG_INDEX
#define CSID_C_WEBUI_AP_LIST_VER              _CFG_WEBUI(0x101c)  //T=str, D-Link GK team record dif.htm ver
#define CSID_C_WEBUI_VPN_CMD                  _CFG_WEBUI(0x101d)	//T=u8, D=0, VPN scenario 0:Site to Site 1:Site to Host 2:Host to Site 3:Host to Host 4:Dynamic VPN
/* "C_WEBUI_VPN_CMD" need 32 rules, so CSID reserve to "0x103d" */
#define CSID_C_WEBUI_UI_FLAG                  _CFG_WEBUI(0x1100)	//T=u8, D=0, for UI save flags
#define CSID_C_WEBUI_UI_SETTING               _CFG_WEBUI(0x1200)	//T=str, for UI save settings
#define	CSID_C_WEBUI_LOGTYPE          	      _CFG_WEBUI(0x1300)	//T=str, D="0000", Log Type for UI
#define	CSID_C_WEBUI_WIZARD_LANG          	  _CFG_WEBUI(0x1400)	//T=u8, D=0, D-link web wizard language select
#define	CSID_C_WEBUI_PICTURE_AUTH        	    _CFG_WEBUI(0x1500)	//T=u8, D=0, D-link picture authentication when login
#define	CSID_C_WEBUI_LOCK_WPS_PIN         	  _CFG_WEBUI(0x1600)	//T=u8, D=0, D-link lock wps button
#define	CSID_C_WEBUI_WEP_KEY_LEN      	      _CFG_WEBUI(0x1700)	//T=u8, D=0, D-link,[0] is 2.4G,[1] is 5G,[2] is guest zone 2.4G, [3] is guest zone 5G
#define	CSID_C_WEBUI_FAKE_CHANNEL      	      _CFG_WEBUI(0x1800)	//T=u8, D=0, D-link 840L channel 11 12 13 is fake
/* for recoed UI rules (each 64 rules) 2000~2040~2080~20c0 */
#define CSID_C_WEBUI_VPNLIST_RECORD_RULE	    _CFG_WEBUI(0x2000)	//T=u32, record the rule sequence for AMTI VPN list, reserve 64
#define CSID_C_WEBUI_VPNPPTPACC_RECORD_RULE	  _CFG_WEBUI(0x2040)	//T=u32, record the rule sequence for AMTI C_PPTPSERV_ACCOUNT_USER list, reserve 64
#define CSID_C_WEBUI_VPNPPTPCLI_RECORD_RULE	  _CFG_WEBUI(0x2080)	//T=u32, record the rule sequence for AMTI C_PPTP_CLI_TUNLNAME list, reserve 64
#define CSID_C_WEBUI_VPNL2TPACC_RECORD_RULE	  _CFG_WEBUI(0x20c0)	//T=u32, record the rule sequence for AMTI C_L2TPSERV_ACCOUNT_USER list, reserve 64
#define CSID_C_WEBUI_VPNL2TPNAM_RECORD_RULE	  _CFG_WEBUI(0x2100)	//T=u32, record the rule sequence for AMTI C_L2TP_CLI_TUNLNAME list, reserve 64
#define CSID_C_WEBUI_GRETNAM_RECORD_RULE	    _CFG_WEBUI(0x2140)	//T=u32, record the rule sequence for AMTI C_GRE_TNAME list, reserve 64
#define CSID_C_WEBUI_LOADBALANCE_RECORD_RULE  _CFG_WEBUI(0x2180)	//T=u32, record the rule sequence for AMTI load balance policy list, reserve 64
#define CSID_C_WEBUI_PKBOTHSIP0_RECORD_RULE   _CFG_WEBUI(0x21c0)	//T=u32, record the rule sequence for AMTI C_PKFILTER_BOTH_RULE_SIP0 list, reserve 64
#define CSID_C_WEBUI_URLBLOCK_RECORD_RULE     _CFG_WEBUI(0x2200)	//T=u32, record the rule sequence for AMTI C_URLBLOCK_RULE_URL list, reserve 64
#define CSID_C_WEBUI_MACCTL_RECORD_RULE       _CFG_WEBUI(0x2240)	//T=u32, record the rule sequence for AMTI C_MACCTL_RULE_MAC list, reserve 64
#define CSID_C_WEBUI_URLBLOCKEXT_RECORD_RULE  _CFG_WEBUI(0x2280)	//T=u32, record the rule sequence for AMTI C_URLBLOCK_FILE_EXTENSION_RULE_LIST, reserve 64
#define CSID_C_WEBUI_SSLVPNAPP_RECORD_RULE    _CFG_WEBUI(0x22c0)	//T=u32, record the rule sequence for AMTI C_SSLVPN_APPLICATION_NAME, reserve 64
#define CSID_C_WEBUI_SSLVPNEXT_RECORD_RULE    _CFG_WEBUI(0x2300)	//T=u32, record the rule sequence for AMTI C_SSLVPN_EXTENDER_NAME, reserve 64
#define CSID_C_WEBUI_SSLVPNUSER_RECORD_RULE   _CFG_WEBUI(0x2340)	//T=str, record the rule sequence for AMTI C_SSL_VPN_USER_TYPE, reserve 64

/* Status Items */

#define CSID_S_WEBUI_CONFIRM		       	_STA_WEBUI(0x0010) //T=u8,  D=0
#define CSID_S_WEBUI_LEAVE				      _STA_WEBUI(0x0020) //T=u8,  D=0
#define CSID_S_WEBUI_CONFIRM_FAIL		    _STA_WEBUI(0x0030) //T=u8,  D=0
#define CSID_S_WEBUI_CONFIRM_FAIL_INDEX _STA_WEBUI(0x0040) //T=u8,  D=0
#define	CSID_S_WEBUI_CONFIRM_FAIL_NAME	_STA_WEBUI(0x0050) //T=str, D="", Confirm fail dev.
#define CSID_S_WEBUI_USERREBOOT         _STA_WEBUI(0x0060) //T=u32, D=0, max transmission unit
#define CSID_S_WEBUI_SPILT_RESULT       _STA_WEBUI(0x0070) //T=u8,  D=0, paggy need for D-Link use
#define CSID_S_WEBUI_FWDATE       		  _STA_WEBUI(0x0080) //T=str, D=""
#define CSID_S_WEBUI_LANGUAGE       		_STA_WEBUI(0x0090) //T=str, D="US"
#define	CSID_S_WEBUI_ODM_STRINGS       	_STA_WEBUI(0x0100) //T=str, D="", ODM Strings

#define CSID_S_WEBUI_FW_QUERY_REGION_NAME     _STA_WEBUI(0x0110) //T=str, the region name of fw query
#define CSID_S_WEBUI_FW_QUERY_ALTERED         _STA_WEBUI(0x0111) //T=u32, D=0, fw query check alter
#define CSID_S_WEBUI_FW_QUERY_VER             _STA_WEBUI(0x0112) //T=str, the fw version of fw query
#define CSID_S_WEBUI_FW_QUERY_VER_MATCH       _STA_WEBUI(0x0113) //T=u32, the matching flag between current fw version and fw query version
#define CSID_S_WEBUI_FW_QUERY_DATE            _STA_WEBUI(0x0114) //T=str, the release date of fw query
#define CSID_S_WEBUI_FW_QUERY_STATUS          _STA_WEBUI(0x0115) //T=u32, D=0, fw query auto download status
#define CSID_S_WEBUI_FW_QUERY_PERCENTAGE      _STA_WEBUI(0x0116) //T=u32, D=0, fw download and upgrade's Percentage
#define CSID_S_WEBUI_FW_QUERY_TIME            _STA_WEBUI(0x0117) //T=u32, D=0, fw download and upgrade's Sec
#define CSID_S_WEBUI_FW_UPGRADE_FLAG          _STA_WEBUI(0x0118) //T=u32, D=0, fw upgrade flag to count progress
#define CSID_S_WEBUI_FW_UPGRADE_START         _STA_WEBUI(0x0119) //T=u8, D=0, 1:start upg

#define CSID_S_WEBUI_FW_QUERY_DW_URL          _STA_WEBUI(0x0120) //T=str, the fw download url of fw query

#define CSID_S_WEBUI_FW_UPGRADE_STATUS        _STA_WEBUI(0x0121) //T=u32, the status of firmware upgrade

#define CSID_S_WEBUI_FW_QUERY_URL             _STA_WEBUI(0x0122) //T=str, the url of fw query
/* 0x0120~0x012F are reserved */

#define CSID_S_WEBUI_DET_DEVICE               _STA_WEBUI(0x0130) //T=u8, D=0, DIR-508L, 1 is click from phone UI to pc UI
#define CSID_S_WEBUI_URL_FOR_F5               _STA_WEBUI(0x0131) //T=str, DIR-508L want click F5 remain in the same page, not goto index.htm
/* 0x0131~0x013F are reserved */

#endif //ifndef _CSID_WEBUI_H
